use embassy_time::{Instant, Duration};
use atat::{
    atat_derive::{AtatCmd, AtatResp, AtatUrc},
    asynch::Client,
    ResponseChannel,
    Config,
};
use embedded_io_async::Write;
use heapless::String;

#[derive(Clone, AtatResp)]
pub struct NoResponse;

#[derive(Clone, AtatCmd)]
#[at_cmd("+CSGT", NoResponse)]
pub struct SetGreetingText<'a> {
    #[at_arg(position = 0, len = 32)]
    pub text: &'a str,
}

#[derive(Clone, AtatCmd)]
#[at_cmd("AT", GreetingText)]
pub struct GetGreetingText;

#[derive(Clone, AtatResp)]
pub struct GreetingText {
    #[at_arg(position = 0)]
    pub text: String<64>,
}

#[derive(Clone, AtatCmd)]
#[at_cmd("+CGMI", ManufacturerIdText)]
pub struct GetManufacturerId;

#[derive(Clone, AtatResp)]
pub struct ManufacturerIdText {
    #[at_arg(position = 0)]
    pub text: String<32>,
}

#[derive(Clone, AtatResp)]
pub struct MessageWaitingIndication;

#[derive(Clone, AtatUrc)]
pub enum Urc {
    #[at_urc("+UMWI")]
    MessageWaitingIndication(MessageWaitingIndication),
}

type ModemClient<'a, W> = Client<'a, W, 256>;

pub fn setup_modem<W>(tx: W, res_channel: &'_ ResponseChannel<256>) -> ModemClient<'_, W>
where W: Write {
    let config = Config::new();
    let client: ModemClient<W> = Client::new(tx, res_channel, config);
    client
}
